/* ----------------------------------------------------------------------------
 *
 * Raima Database Manager
 *
 * Copyright (c) 2012 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 *
 * ----------------------------------------------------------------------------
 */

#include <stdio.h>

/* rdm.h is required in all files using the RDM Database */
#include "rdm.h"

/* core08_db.h was generated from the DDL file core08_db.ddl by the ddlp.
 * utility.
 */
#include "core08_db.h"

/* core08_db_dbd.h was generated from the DDL
 * file core08_db.ddl by the ddlp utility.  It is required to use the d_open_ptr API
 */
#include "core08_db_dbd.h"

/* EXAMPLE - Core for C */
/* Each time you run this example:
 *   1) The database is initialized (all existing contents removed)
 *   2) Several new offices are added to the NA database
 *   3) Several new offices are added to the EMEA database
 *   4) Data is queried from the NA office
 *   5) Data is queried from the EMEA office
 *
 *
 * The DDL (Database Definition Language) is in sales_db.ddl. The database
 * schema is embedded into the application and the database will be created 
 * the first time the application is run.  Subsequent runs will use the now
 * existing database
 *
 * The following RDM Functions are illustrated:
 *      d_opentask/d_closetask
 *      d_open_ptr/d_iopen_ptr/d_close
 *      d_dbnum
 *      d_initialize
 *      d_fillnew
 *      d_keyfrst/d_keynext
 *      d_recread
 * They are fully documented in http://docs.raima.com/rdm/.
 *
 * Each of these functions returns an integer status code, where the
 * value S_OKAY indicates a successful call.
 *
 * For simplicity, this example does not check all return codes, but good
 * programming practices would dictate that they are checked after each
 * RDM call.
 *
 * The actual databases will be stored in a directories named 'na_sales_db' and
 * 'emea_sales_db' in the project directory.
 */

int32_t display_offices(
    DB_TASK *task,
    int32_t  dbn)
{
    int32_t       rdm_status;
    struct office office_rec;

    for (rdm_status = d_keyfrst (OFFICE_NAME, task, dbn);
         rdm_status == S_OKAY;
         rdm_status = d_keynext (OFFICE_NAME, task, dbn))
    {
        /* Read and display the current person record */
        d_recread(&office_rec, task, dbn);
        printf("%s\n", office_rec.name);
    }

    /* Expect rdm_status to be S_NOTFOUND when we exit the loop */
    if(rdm_status == S_NOTFOUND)
    {
        rdm_status = S_OKAY;
    }
    return rdm_status;
}

/* =======================================================================
*/
int32_t main(void)
{
    DB_TASK       *task;
    int32_t        status;
    struct office  office_rec;
    int32_t        na_dbn;
    int32_t        emea_dbn;

    /* create the context for the database operations */
    d_opentask(&task);

    /* Open the NA database for exclusive access, this will create the database
     * dictionary (na_sales_db.dbd file if it does not already exist.  Notice we
     * use the 'na_' prefix which will create a database called 'na_sales_db'
     * using the schema defined in sales_db_dbd.
     */
    status = d_open_ptr("na_core08_db", "x", &core08_db_dbd, sizeof(core08_db_dbd), task);
    if (status != S_OKAY)
    {
        fprintf (stderr, "Exclusive open failed, error = %s: %s\n", d_errorname(status), d_errorstr(status) );
        d_closetask (task);
        return (1);
    }

    /* Get the Database Number (dbn) for the NA Sales Database */
    na_dbn = d_dbnum("na_core08_db", task);

    /* Incrementally open the EMEA database, this will create the database
     * dictionary (emea_sales_db.dbd file if it does not already exist.
     *  Notice we use the 'emea_' prefix which will create a database called
     * 'emea_sales_db' using the schema defined in sales_db_dbd.
     */
    status = d_iopen_ptr("emea_core08_db", &core08_db_dbd, sizeof(core08_db_dbd), task);
    if (status != S_OKAY)
    {
        fprintf (stderr, "Incremental open failed, error = %s: %s\n", d_errorname(status), d_errorstr(status) );
        d_close(task);
        d_closetask (task);
        return (1);
    }

    /* Get the Database Number (dbn) for the NA Sales Database */
    emea_dbn = d_dbnum("emea_core08_db", task);

    /* Initialize the databases.  Initializing will create the data files if 
     * they do not exist.  If the the data files do already exist they will be
     * removed and new empty files will be created.
     */
    d_initialize(task, na_dbn);
    d_initialize(task, emea_dbn);

    /* create records in the NA database */
    strncpy (office_rec.name, "Seattle", SIZEOF_OFFICE_NAME);
    status = d_fillnew (OFFICE, &office_rec, task, na_dbn);

    strncpy (office_rec.name, "Boise", SIZEOF_OFFICE_NAME);
    status = d_fillnew (OFFICE, &office_rec, task, na_dbn);

    strncpy (office_rec.name, "San Francisco", SIZEOF_OFFICE_NAME);
    status = d_fillnew (OFFICE, &office_rec, task, na_dbn);

    strncpy (office_rec.name, "Dallas", SIZEOF_OFFICE_NAME);
    status = d_fillnew (OFFICE, &office_rec, task, na_dbn);

    /* create records in the EMEA database */
    strncpy (office_rec.name, "Paris", SIZEOF_OFFICE_NAME);
    status = d_fillnew (OFFICE, &office_rec, task, emea_dbn);

    strncpy (office_rec.name, "London", SIZEOF_OFFICE_NAME);
    status = d_fillnew (OFFICE, &office_rec, task, emea_dbn);

    strncpy (office_rec.name, "Dublin", SIZEOF_OFFICE_NAME);
    status = d_fillnew (OFFICE, &office_rec, task, emea_dbn);

    strncpy (office_rec.name, "Zurich", SIZEOF_OFFICE_NAME);
    status = d_fillnew (OFFICE, &office_rec, task, emea_dbn);

    strncpy (office_rec.name, "Madrid", SIZEOF_OFFICE_NAME);
    status = d_fillnew (OFFICE, &office_rec, task, emea_dbn);

    /* Display all offices in North America */
    printf("\nNorth American Offices\n");
    display_offices(task, na_dbn);

    /* Display All offices in EMEA */
    printf("\nEMEA Offices\n");
    display_offices(task, emea_dbn);

    /* close the database */
    d_close (task);
    d_closetask (task);
    
    printf ("\nPress <Enter> to continue . . .\n");
    getchar ();
}
